@@ -19,14 +19,15 @@
 
    Authors: Felix Bellaby, Owen Taylor */
 
-#ifdef HAVE_CONFIG_H
 #include <config.h>
-#endif
 
 #include <string.h>
 
-#include <gnome.h>
+#include <glib/gi18n.h>
+
 #include <libgnomeui/gnome-client.h>
+#include <libgnomeui/gnome-help.h>
+#include <libgnomeui/gnome-ui-init.h>
 
 #include <gconf/gconf-client.h>
 #include "headers.h"
@@ -159,7 +160,8 @@ show_message_dialog (GtkWidget *parent)
 			GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 			GTK_STOCK_SAVE, SESSION_RESPONSE_SAVE,
 			NULL);
-	gtk_dialog_set_default_response (GTK_DIALOG (dlg), 1);
+	gtk_dialog_set_default_response (GTK_DIALOG (dlg),
+					 SESSION_RESPONSE_SAVE);
 	gtk_window_set_title (GTK_WINDOW (dlg), "");
 
 	response = gtk_dialog_run (GTK_DIALOG (dlg));
@@ -239,8 +241,8 @@ capplet_build (void)
   /* Frame for manually started programs */
   startup_list = startup_list_read ();
 
-  vbox = gtk_vbox_new (FALSE, GNOME_PAD);
-  gtk_container_set_border_width (GTK_CONTAINER (vbox), GNOME_PAD);
+  vbox = gtk_vbox_new (FALSE, 8);
+  gtk_container_set_border_width (GTK_CONTAINER (vbox), 8);
  
   /*non-session managed startup programs */
   a = gtk_alignment_new (0.0, 0.5, 0.0, 0.0);
@@ -248,7 +250,7 @@ capplet_build (void)
   label = gtk_label_new_with_mnemonic (_("Additional startup _programs:"));
   gtk_container_add (GTK_CONTAINER (a), label);
 
-  hb = gtk_hbox_new (FALSE, GNOME_PAD);
+  hb = gtk_hbox_new (FALSE, 8);
   gtk_box_pack_start (GTK_BOX (vbox), hb, TRUE, TRUE,0);
 
   sw = gtk_scrolled_window_new (NULL, NULL);
@@ -265,7 +267,7 @@ capplet_build (void)
   gtk_label_set_mnemonic_widget (GTK_LABEL (label),
 		  		 GTK_WIDGET (startup_view));
   startup_sel = gtk_tree_view_get_selection (startup_view);
-  gtk_tree_selection_set_mode (startup_sel, GTK_SELECTION_SINGLE);
+  gtk_tree_selection_set_mode (startup_sel, GTK_SELECTION_BROWSE);
   g_signal_connect (G_OBJECT (startup_sel), "changed",
                     (GCallback) selection_changed_cb, startup_view);
   renderer = gtk_cell_renderer_toggle_new ();
@@ -273,27 +275,39 @@ capplet_build (void)
                                                      "active", 1,
                                                      "activatable", 3,
                                                      NULL);
+  gtk_tree_view_column_set_sort_column_id (column, 1);
   gtk_tree_view_append_column (startup_view, column);
   g_signal_connect (G_OBJECT (renderer), "toggled",
                     (GCallback) startup_enabled_toggled, NULL);
   renderer = gtk_cell_renderer_text_new ();
   column = gtk_tree_view_column_new_with_attributes (_("Program"), renderer,
-                                                     "text", 2, NULL);
+                                                     "markup", 2,
+						     NULL);
+  g_object_set (renderer, "ellipsize", PANGO_ELLIPSIZE_END, NULL);
+  gtk_tree_view_column_set_sort_column_id (column, 2);
   gtk_tree_view_append_column (startup_view, column);
 
   gtk_tree_view_set_search_column (startup_view, 2);
+  gtk_tree_view_set_rules_hint (startup_view, TRUE);
 
   gtk_container_add (GTK_CONTAINER (sw), GTK_WIDGET (startup_view));
 
-  util_vbox = gtk_vbox_new (FALSE, GNOME_PAD_SMALL);
+  util_vbox = gtk_vbox_new (FALSE, 4);
   gtk_box_pack_start (GTK_BOX (hb), util_vbox, FALSE, FALSE, 0);
 
   /* Add/Edit/Delete buttons */
-  button = gtk_button_new_from_stock (GTK_STOCK_NEW);
+  button = gtk_button_new_from_stock (GTK_STOCK_ADD);
   g_signal_connect (button, "clicked",
 		    G_CALLBACK (add_startup_cb), dlg);
   gtk_box_pack_start (GTK_BOX (util_vbox), button, FALSE, FALSE, 0);
-  
+
+  button = gtk_button_new_from_stock (GTK_STOCK_REMOVE);
+  g_signal_connect (button, "clicked",
+		    G_CALLBACK (delete_startup_cb), NULL);
+  gtk_box_pack_start (GTK_BOX (util_vbox), button, FALSE, FALSE, 0);
+  gtk_widget_set_sensitive (button, FALSE);
+  g_object_set_data (G_OBJECT (startup_view), "delete", button);
+
   button = gtk_button_new_from_stock (GTK_STOCK_EDIT);
   g_signal_connect (button, "clicked",
 		    G_CALLBACK (edit_startup_cb), dlg);
@@ -301,13 +315,6 @@ capplet_build (void)
   gtk_widget_set_sensitive (button, FALSE);
   g_object_set_data (G_OBJECT (startup_view), "edit", button);
   
-  button = gtk_button_new_from_stock (GTK_STOCK_DELETE);
-  g_signal_connect (button, "clicked",
-		    G_CALLBACK (delete_startup_cb), NULL);
-  gtk_box_pack_start (GTK_BOX (util_vbox), button, FALSE, FALSE, 0);
-  gtk_widget_set_sensitive (button, FALSE);
-  g_object_set_data (G_OBJECT (startup_view), "delete", button);
-
   /* Button for running session-properties */
   a = gtk_alignment_new (0.0, 0.5, 0.0, 0.0);
   gtk_box_pack_start (GTK_BOX (vbox), a, FALSE, FALSE, 0);
@@ -322,16 +329,16 @@ capplet_build (void)
   gtk_notebook_append_page (GTK_NOTEBOOK (notebook), vbox, label);
  
   /* Options page - Autosave, Save session */
-  vbox = gtk_vbox_new (FALSE, GNOME_PAD);
+  vbox = gtk_vbox_new (FALSE, 8);
   
   util_vbox = gtk_vbox_new (FALSE, 0);
   gtk_box_pack_start (GTK_BOX (vbox), util_vbox, FALSE, FALSE, 0);
-  gtk_container_set_border_width (GTK_CONTAINER (vbox) , GNOME_PAD);
+  gtk_container_set_border_width (GTK_CONTAINER (vbox) , 8);
 
   /* Autosave */
   a = gtk_alignment_new (0.0, 0.5, 0.0, 0.0);
   gtk_box_pack_start (GTK_BOX (util_vbox), a, FALSE, FALSE, 0);
-  autosave_button = gtk_check_button_new_with_mnemonic (_("Automatically save chan_ges to session"));
+  autosave_button = gtk_check_button_new_with_mnemonic (_("_Automatically remember running applications when logging out"));
   gtk_container_add (GTK_CONTAINER (a), autosave_button);
   g_object_set_data (G_OBJECT (autosave_button), "key", AUTOSAVE_MODE_KEY);
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (autosave_button),
@@ -346,7 +353,7 @@ capplet_build (void)
   hbox = gtk_hbutton_box_new ();
   gtk_box_pack_start (GTK_BOX (util_vbox), hbox, FALSE, FALSE, 0);
 
-  button = gtk_button_new_with_label (_("Save the current session"));
+  button = gtk_button_new_with_mnemonic (_("_Remember currently running applications"));
   gtk_button_set_image (GTK_BUTTON (button), gtk_image_new_from_stock (GTK_STOCK_SAVE, GTK_ICON_SIZE_BUTTON));
   gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 0);
   g_signal_connect (G_OBJECT (button), "clicked", G_CALLBACK (save_session_cb), NULL);
@@ -486,7 +493,7 @@ main (int argc, char *argv[])
   /* Don't restart the capplet, it's the place where the session is saved. */
   gnome_client_set_restart_style (gnome_master_client (), GNOME_RESTART_NEVER);
 
-  gtk_window_set_default_icon_name ("gnome-session");
+  gtk_window_set_default_icon_name ("session-properties");
 
   client = gconf_client_get_default ();
   gconf_client_add_dir (client, GSM_GCONF_CONFIG_PREFIX, GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
@@ -494,13 +501,13 @@ main (int argc, char *argv[])
   master_client = gnome_master_client ();
   if (! GNOME_CLIENT_CONNECTED (master_client)) {
     g_printerr (_("could not connect to the session manager\n"));
-    exit (1);
+    return 1;
   }
 
   protocol = gsm_protocol_new (master_client);
   if (!protocol) {
     g_printerr (_("session manager does not support GNOME extensions\n"));
-    exit (1);
+    return 1;
   }
 
   g_signal_connect (gnome_master_client (), "die", G_CALLBACK (gtk_main_quit), NULL);
